///////////////////////////////////////////////////////////////////////
//  CookieTest.cpp: Testing program for CookieManager class as well as
//  Cookie Class
//
//  Created by Babu George Padamadan
//  Created on Oct 2000
//  Contact : bgeorge@wol.com or babu_george@hotmail.com
//
///////////////////////////////////////////////////////////////////////

#include "CookieManager.h"
#include <iostream>

using std::cout;

int main(int argc, char* argv[])
{
    // Test Cases based on the Examples given in the Specifications
    // More Test Cases are available in the CookieTest1.cpp in the ftp site
    bool bSecure = false;
    string strCookie;

    // Version 0 Specification (Netscape Specification)
    cout << "-- Version 0 Specification (Netscape Specification) --";
    cout << "\nExample 1 \n";

    // Example 1.1
    CookieManager objOldSpecMgr("www.foo.com");
    strCookie = "Set-Cookie: CUSTOMER=WILE_E_COYOTE; path=/; " +
        string("expires=Friday, 09-Nov-2001 23:12:12 GMT");
    cout << "Adding : " << strCookie << "\n";
    objOldSpecMgr.addCookie (strCookie);

    // Results
    cout << "Result - Request Path=/ : \n";
    cout << objOldSpecMgr.getCookieString ("/", bSecure);

    // Example 1.2
    strCookie = "Set-Cookie: PART_NUMBER=ROCKET_LAUNCHER_0001; path=/";
    cout << "\nAdding : " << strCookie << "\n";
    objOldSpecMgr.addCookie (strCookie);
    
    // Results
    cout << "Result - Request Path=/ : \n";
    cout << objOldSpecMgr.getCookieString ("/", bSecure) << "\n";

    // Example 1.3
    strCookie = "Set-Cookie: SHIPPING=FEDEX; path=/foo";
    cout << "Adding : " << strCookie << "\n";
    objOldSpecMgr.addCookie (strCookie);

    // Results
    cout << "Result - Request Path=/ : \n";
    cout << objOldSpecMgr.getCookieString ("/", bSecure) << "\n";
    cout << "Result - Requst Path = /foo : \n";
    cout << objOldSpecMgr.getCookieString ("/foo", bSecure) << "\n";

    // Example 2.1
    cout << " Example 2 \n";
    objOldSpecMgr.clearAll();
    strCookie = "Set-Cookie: PART_NUMBER=ROCKET_LAUNCHER_0001; path=/";
    cout << "Adding : " << strCookie << "\n";
    objOldSpecMgr.addCookie (strCookie);

    // Results
    cout << "Result - Request Path=/ : \n";
    cout << objOldSpecMgr.getCookieString ("/", bSecure) << "\n";
    
    // Example 2.2
    strCookie = "Set-Cookie: PART_NUMBER=RIDING_ROCKET_0023; path=/ammo";
    cout << "Adding : " << strCookie << "\n";
    objOldSpecMgr.addCookie (strCookie);

    // Results
    cout << "Result - Request Path=/ammo : \n";
    cout << objOldSpecMgr.getCookieString ("/ammo", bSecure) << "\n";

    //--------------------------------------------------------------//
    // RFC 2109 Specification
    cout << "\n\n-----  RFC 2109 Specification ----- " << "\n";
    CookieManager objRFC2109Mgr;
    objRFC2109Mgr.setDomain("www.acme.com");
    cout << " Example 1 \n";

    // Example 1.1
    strCookie = "Set-Cookie: CUSTOMER=\"WILE_E_COYOTE\"; Version=\"1\"; " +
        string("Path=\"/acme\"");
    cout << "Adding : " << strCookie << "\n";
    objRFC2109Mgr.addCookie (strCookie);

    // Results
    cout << "Result - Request Path=/acme/pickitem : \n";
    cout << objRFC2109Mgr.getCookieString ("/acme/pickitem", bSecure) << "\n";
    
    // Example 1.2
    strCookie = "Set-Cookie: Part_Number=\"Rocket_Launcher_0001\"; " +
        string("Version=\"1\"; Path=\"/acme\"");
    cout << "Adding : " << strCookie << "\n";
    objRFC2109Mgr.addCookie (strCookie);

    // Results
    cout << "Result - Request Path=/acme/shipping : \n";
    cout << objRFC2109Mgr.getCookieString ("/acme/shipping", bSecure) << "\n";

    // Example 1.3
    strCookie = "Set-Cookie: Shipping=\"FedEx\"; Version=\"1\"; " +
        string("Path=\"/acme\"");
    cout << "Adding : " << strCookie << "\n";
    objRFC2109Mgr.addCookie (strCookie);

    // Results
    cout << "Result - Request Path=/acme/process : \n";
    cout << objRFC2109Mgr.getCookieString ("/acme/process", bSecure) << "\n";

    // Example 2
    cout << " Example 2 \n";
    objRFC2109Mgr.clearAll();

    // Example 2.1
    strCookie = "Set-Cookie: Part_Number=\"Rocket_Launcher_0001\"; " +
        string("Version=\"1\"; Path=\"/acme\"");
    cout << "Adding : " << strCookie << "\n";
    objRFC2109Mgr.addCookie (strCookie);
    strCookie = "Set-Cookie: Part_Number=\"Riding_Rocket_0023\"; " +
        string("Version=\"1\"; Path=\"/acme/ammo\"");
    cout << "Adding : " << strCookie << "\n";
    objRFC2109Mgr.addCookie (strCookie);

    // Results
    cout << "Result - Request Path=/acme/ammo/pickitem : \n";
    cout << objRFC2109Mgr.getCookieString ("/acme/ammo/pickitem", bSecure) << "\n";
    cout << "Result - Request Path=/acme/parts/ : \n";
    cout << objRFC2109Mgr.getCookieString ("/acme/parts/", bSecure) << "\n";

    return 0;
}

