///////////////////////////////////////////////////////////////////////
//  CookieTest.cpp: Testing program for CookieManager class as well as
//  Cookie Class
//
//  Created by Babu George Padamadan
//  Created on Oct 2000
//  Contact : bgeorge@wol.com or babu_george@hotmail.com
//
///////////////////////////////////////////////////////////////////////

#include "CookieManager.h"
#include <iostream>

using std::cout;

int main(int argc, char* argv[])
{
    string strCookie;

    // Old Specification
    cout << " -----  Old Specification ----- " << "\n";
    CookieManager objOldSpecMgr("www.acme.com");
    strCookie = "set-cookie: PRODUCT=newitem1; domain=.acme.com; path=/start";
    cout << "Adding : " << strCookie << "\n";
    objOldSpecMgr.addCookie (strCookie);
    strCookie = "PRODUCT=newitem2; domain=.acme.com; path=/start";
    cout << "Adding : " << strCookie << "\n";
    objOldSpecMgr.addCookie (strCookie);

    // Get the Cookie string to be sent to the server
    strCookie = objOldSpecMgr.getCookieString ("/", false);

    // Results
    cout << "Results : \n" << strCookie << "\n";

    // Test 2
    objOldSpecMgr.clearSession();
    strCookie = "set-cookie: PRODUCT1=newitem1; domain=.acme.com; path=/start";
    cout << "Adding : " << strCookie << "\n";
    objOldSpecMgr.addCookie (strCookie);
    strCookie = "set-cookie: PRODUCT2=newitem2; domain=.acme.com; path=/start";
    cout << "Adding : " << strCookie << "\n";
    objOldSpecMgr.addCookie (strCookie);
    
    // Get the Cookie string to be sent to the server
    strCookie = objOldSpecMgr.getCookieString ("/", false);

    // Results
    cout << "Results : \n" << strCookie << "\n";

    // Test 3
    objOldSpecMgr.clearSession();
    strCookie = "PRODUCT1=newitem1; domain=.acme.com; path=/start";
    cout << "Adding : " << strCookie << "\n";
    objOldSpecMgr.addCookie (strCookie);
    strCookie = "PRODUCT2=newitem2; expires=Sunday, 01-Oct-2000 00:00:00 GMT; " +
        string("domain=.acme.com; path=/start; ");
    cout << "Adding : " << strCookie << "\n";
    objOldSpecMgr.addCookie (strCookie);

    // Get the Cookie string to be sent to the server
    strCookie = objOldSpecMgr.getCookieString ("/", false);

    // Results
    cout << "-- Added Expired Cookie -- PRODUCT2=newitem2 \n";
    cout << "Results : \n" << strCookie << "\n";

    // Test 4
    objOldSpecMgr.clearSession();
    strCookie = "PRODUCT=newitem1; CHECK=1212; domain=.acme.com; path=/start";
    cout << "Adding : " << strCookie << "\n";
    objOldSpecMgr.addCookie (strCookie);
    strCookie = "PRODUCT=newitem2; domain=.acme.com; path=/start";
    cout << "Adding : " << strCookie << "\n";
    objOldSpecMgr.addCookie (strCookie);

    // Get the Cookie string to be sent to the server
    strCookie = objOldSpecMgr.getCookieString ("/", false);

    // Results
    cout << "Results : \n" << strCookie << "\n";

    // Test 5
    objOldSpecMgr.clearSession();
    strCookie = "PRODUCT=newitem1; CHECK=1212; domain=.acme.com; path=/start";
    cout << "Adding : " << strCookie << "\n";
    objOldSpecMgr.addCookie (strCookie);
    strCookie = "PRODUCT=newitem2; domain=.acme.com; path=/start; " +
        string ("expires=Sunday, 01-Oct-2000 00:00:00 GMT;");
    cout << "Adding : " << strCookie << "\n";
    objOldSpecMgr.addCookie (strCookie);

    // Get the Cookie string to be sent to the server
    strCookie = objOldSpecMgr.getCookieString ("/", false);

    // Results
    cout << "Results : \n" << strCookie << "\n";


    //--------------------------------------------------------------------------------//
    // RFC 2109 Specification
    cout << "\n\n-----  RFC 2109 Specification ----- " << "\n";
    CookieManager objRFC2109Mgr("www.acme.com");
    strCookie = "PRODUCT=\"newitem1\"; domain=\".acme.com\"; path=\"/start\"; version=\"1\"";
    //strCookie = "PRODUCT=newitem1; domain=.acme.com; path=/start; version=";
    cout << "Adding : " << strCookie << "\n";
    objRFC2109Mgr.addCookie (strCookie);
    strCookie = "PRODUCT=\"newitem2\"; domain=\".acme.com\"; path=\"/start\"; version=\"1\"";
    cout << "Adding : " << strCookie << "\n";
    objRFC2109Mgr.addCookie (strCookie);

    // Get the Cookie string to be sent to the server
    strCookie = objRFC2109Mgr.getCookieString ("/", false);

    // Results
    cout << "Results : \n" << strCookie << "\n";

    // Test 2
    objRFC2109Mgr.clearSession();
    strCookie = "PRODUCT1=newitem1; domain=.acme.com; path=/start; version=1";
    cout << "Adding : " << strCookie << "\n";
    objRFC2109Mgr.addCookie (strCookie);
    strCookie = "PRODUCT2=newitem2; domain=.acme.com; path=/start; version=1";
    cout << "Adding : " << strCookie << "\n";
    objRFC2109Mgr.addCookie (strCookie);

    // Get the Cookie string to be sent to the server
    strCookie = objRFC2109Mgr.getCookieString ("/", false);

    // Results
    cout << "Results : \n" << strCookie << "\n";

    // Test 3
    objRFC2109Mgr.clearSession();
    //strCookie = "PRODUCT1=newitem1; domain=.acme.com; path=/start; version=1";
    strCookie = "PRODUCT1=\"newitem1\"; domain=\".acme.com\"; path=\"/start\"; version=\"1\"";
    cout << "Adding : " << strCookie << "\n";
    objRFC2109Mgr.addCookie (strCookie);
    strCookie = "PRODUCT2=newitem2; max-age=0; " +
        string("domain=.acme.com; path=/; version=1");
    cout << "Adding : " << strCookie << "\n";
    objRFC2109Mgr.addCookie (strCookie);

    // Get the Cookie string to be sent to the server
    strCookie = objRFC2109Mgr.getCookieString ("/start", false);

    // Results
    cout << "-- Added Expired Cookie -- PRODUCT2=newitem2 \n";
    cout << "Results : \n" << strCookie << "\n";

    // Test 4
    objRFC2109Mgr.clearAll();
    //strCookie = "PRODUCT1=newitem1; domain=.acme.com; path=/start; version=1";
    strCookie = "PRODUCT1=\"newitem1\"; domain=\".acme.com\"; path=\"/start\"; version=\"1\"";
    cout << "Adding : " << strCookie << "\n";
    objRFC2109Mgr.addCookie (strCookie);
    strCookie = "PRODUCT1=\"newitem1\"; domain=\".acme.com\"; path=\"/start/test\"; version=\"1\"";
    cout << "Adding : " << strCookie << "\n";
    objRFC2109Mgr.addCookie (strCookie);
    strCookie = "PRODUCT1=\"newitem1\"; domain=\".acme.com\"; path=\"/\"; version=\"1\"";
    cout << "Adding : " << strCookie << "\n";
    objRFC2109Mgr.addCookie (strCookie);

    // Get the Cookie string to be sent to the server
    //strCookie = objRFC2109Mgr.getCookieString ("/start/test", false);
    strCookie = objRFC2109Mgr.getCookieString ("/start", false);

    // Results
    cout << "-- Added Expired Cookie -- PRODUCT2=newitem2 \n";
    cout << "Results : \n" << strCookie << "\n";

    // Test 4
    objRFC2109Mgr.clearAll();
    //strCookie = "PRODUCT1=newitem1; domain=.acme.com; path=/start; version=1";
    strCookie = "PRODUCT1=\"newitem1\"; secure; version=\"1\"";
    cout << "Adding : " << strCookie << "\n";
    objRFC2109Mgr.addCookie (strCookie);
    strCookie = "PRODUCT2=\"newitem2\"; secure; version=\"1\"";
    cout << "Adding : " << strCookie << "\n";
    objRFC2109Mgr.addCookie (strCookie);
    strCookie = "PRODUCT1=\"newitem1\"; domain=\".acme.com\"; path=\"/\"; version=\"1\"";
    cout << "Adding : " << strCookie << "\n";
    objRFC2109Mgr.addCookie (strCookie);

    // Get the Cookie string to be sent to the server
    //strCookie = objRFC2109Mgr.getCookieString ("/start", true);
    strCookie = objRFC2109Mgr.getCookieString ("/start", false);

    // Results
    cout << "-- Added Expired Cookie -- PRODUCT2=newitem2 \n";
    cout << "Results : \n" << strCookie << "\n";
    
    return 0;
}

